/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
/** @file SprObject.h
	s^NX̊{NX
 */
#ifndef SPR_OBJECT_H
#define SPR_OBJECT_H
#include "../SprBase.h"
#include <iosfwd>

namespace Spr{;

struct ObjectIf;
class IfInfo;
class UTTypeDesc;

/**	Factory (IuWFNg𐶐NX)̊{NX
*/
class FactoryBase:public UTRefCount{
public:
	virtual ~FactoryBase(){}
	///	쐬IfInfo
	virtual const IfInfo* GetIfInfo() const =0;
	///	iĩIuWFNg desc  parent̎qƂĐD
	virtual ObjectIf* Create(const void* desc, ObjectIf* parent) =0;
};

/**	C^tF[X̌^NX̊{NXDNXp֌WĂāCDCAST()ȂǂpD
1̃C^tF[XNXɕt1̃CX^XłD	*/
class IfInfo{
public:
	///	IfInfoid NXRD
	int id;
	///	id̍ől
	static int maxId;
	///	desc^ɂĂ̏ڍׂȏ
	UTTypeDesc* desc;
	///	NX
	const char* className;
	///	{NX
	IfInfo** base;

	typedef std::vector< UTRef<FactoryBase> > Factories;
	///	qIuWFNg𐶐NX(t@Ng[)
	Factories factories;
	///	RXgN^
	IfInfo(const char* cn, IfInfo** b);
	///	NX
	virtual const char* ClassName() const =0;
	///	̃C^tF[XɑΉIuWFNg̃AhX
	virtual void* GetSprObject(ObjectIf* intf)const =0;
	///	IuWFNg̃AhX obj ΉC^tF[X擾
	virtual ObjectIf* GetIf(void* obj)const =0;
	///	key pĂ邩ǂ`FbN
	bool Inherit(const char* key) const;
	///	key pĂ邩ǂ`FbN
	bool Inherit(const IfInfo* key) const;
	///	t@Ng(IuWFNg𐶐NX)̓o^
	void RegisterFactory(FactoryBase* f) const ;
	///	w(info)̃IuWFNgt@Ng
	FactoryBase* FindFactory(const IfInfo* info) const;
protected:
	virtual ~IfInfo() {};
};
///	IfInfo̎D1NX1CX^X
template <class T>
class IfInfoImp: public IfInfo{
public:
	IfInfoImp(const char* cn, IfInfo** b): IfInfo(cn, b){}
	virtual const char* ClassName() const { return className; }
	virtual void* CreateInstance() const{ return 0;}
	virtual void* GetSprObject(ObjectIf* intf)const;
	virtual ObjectIf* GetIf(void* obj)const;
};

///	s^NXׂo̐錾D
#define IF_DEF(cls)										\
public:													\
	static IfInfoImp<cls##If> ifInfo;					\
	virtual const IfInfo* GetIfInfo() const {			\
		return &ifInfo;									\
	}													\
	static const IfInfo* GetIfInfoStatic(){				\
		return &ifInfo;									\
	}													\
	static cls##If* GetSelfFromObject(void* o) {		\
		return (cls##If*)GetIfInfoStatic()->GetIf(o);	\
	}													\


///	ׂẴC^tF[XNX̊{NX
struct ObjectIf{
	IF_DEF(Object);
	virtual ~ObjectIf(){}
	///	fobOp̕\
	virtual void Print(std::ostream& os) const =0;	

	///	@name QƃJE^֌W
	//@{
	///
	virtual int AddRef()=0;
	///
	virtual int DelRef()=0;
	///
	virtual int RefCount()=0;
	//@}


	///	@name qIuWFNg
	//@{
	///	qIuWFNg̐
	virtual size_t NChildObject() const =0;
	///	qIuWFNg̎擾
	virtual ObjectIf* GetChildObject(size_t pos) = 0;
	virtual const ObjectIf* GetChildObject(size_t pos) const = 0;
	/**	qIuWFNg̒ǉD̃IuWFNg̎qIuWFNgƂĒǉĂ悢D
		Ⴆ΁CGRFrame̓c[邪CSm[hGRScene̎qłD*/
	virtual bool AddChildObject(ObjectIf* o)=0;
	///	IuWFNg쐬CAddChildObject()ĂԁD
	virtual ObjectIf* CreateObject(const IfInfo* info, const void* desc)=0;
	//@}

	///	@name fXNv^Ə
	//@{
	/**	fXNv^̓ǂݏo(QƔ NULLԂƂ)D
		ꂪĂȂĂCObjectIf::GetDesc()͎Ă邱ƂD	*/
	virtual const void* GetDescAddress() const = 0;
	/**	fXNv^̓ǂݏo(Rs[ s(falseԂ)Ƃ)D
		ObjectIf::GetDescAddress() ĂȂĂC͎Ă邱ƂD	*/
	virtual bool GetDesc(void* desc) const = 0;
	/**	fBXNv^̃TCY	*/
	virtual size_t GetDescSize() const = 0;
	/**	Ԃ̓ǂݏo(QƔ NULLԂƂ)D
		ꂪĂȂĂCObjectIf::GetState()͎Ă邱ƂD	*/
	virtual const void* GetStateAddress() const = 0;
	/**	Ԃ̓ǂݏo(Rs[ s(falseԂ)Ƃ)D
		ObjectIf::GetStateAddress() ĂȂĂC͎Ă邱ƂD	*/
	virtual bool GetState(void* state) const = 0;
	/**	Ԃ̐ݒ	*/
	virtual void SetState(const void* state) = 0;
	/**	Ԃ̃TCY	*/
	virtual size_t GetStateSize() const = 0;
	/**	ubNԌ^ɏ	*/
	virtual void ConstructState(void* m) const = 0;
	/**	Ԍ^ubNɖ߂	*/
	virtual void DestructState(void* m) const = 0;
	//@}
};

///	C^tF[XNXւ̃|C^̔z
struct ObjectIfs: public UTStack<ObjectIf*>{
};

///	OIuWFNg̃C^tF[X
struct NamedObjectIf: public ObjectIf{
	IF_DEF(NamedObject);
	///	O̎擾
	virtual const char* GetName() const =0;
	///	O̐ݒ
	virtual void SetName(const char* n) =0;
};

struct SceneIf;
///	V[Ot\m[h̃C^tF[X
struct SceneObjectIf: NamedObjectIf{
	IF_DEF(SceneObject);
	///	Scene̎擾
	virtual SceneIf* GetScene() =0;
};


///	IuWFNgc[̏ԂɕۑĂNX
struct ObjectStatesIf: public ObjectIf{
	IF_DEF(ObjectStates);

	///	oƂ̎qZ[u邽߂ɕKvȃmۂD
	virtual void AllocateState(ObjectIf* o) = 0;
	///	Ԃ̃
	virtual void ReleaseState(ObjectIf* o) = 0;
	///	Ԃ̃TCY߂
	virtual size_t CalcStateSize(ObjectIf* o) = 0;

	///	ԂZ[uD
	virtual void SaveState(ObjectIf* o) = 0;
	///	Ԃ[hD
	virtual void LoadState(ObjectIf* o) = 0;
};
ObjectStatesIf* CreateObjectStates();

#undef DCAST
#define DCAST(T,p) SprDcastImp<T>(p)

///	C^tF[XNX̃LXg
template <class T> T* SprDcastImp(const ObjectIf* p){
	void* obj = p ? p->GetIfInfo()->GetSprObject((ObjectIf*)p) : NULL;
	return (T*)T::GetSelfFromObject(obj);
}

}

#endif
