/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
/**
 *	@file SprPHJoint.h
 *	@brief ڐG֐߂Ȃ
*/
#ifndef SPR_PHJOINTIf_H
#define SPR_PHJOINTIf_H
#include <Foundation/SprObject.h>

namespace Spr{;

/** \addtogroup gpPhysics */
//@{
/** \defgroup gpJoint WCg*/
//@{

/// S̃fBXNv^
struct PHConstraintDesc{
	/// S̎
	enum ConstraintType{
		CONTACT,
		HINGEJOINT,
		SLIDERJOINT,
		BALLJOINT,
		PATHJOINT,
		SPRING
	} type;
	/// L/tO
	bool bEnabled;
	/// ̂猩֐߂̈ʒuƌX
	Posed posePlug, poseSocket;
	PHConstraintDesc():bEnabled(true){}
};

/// ֐߂̃fBXNv^	<	oǉȂꍇ́Ctypedefƕʖ` FIDesc.cppɒǉŁD
typedef PHConstraintDesc	PHJointDesc;

/// 1֐߂̃fBXNv^
struct PHJoint1DDesc : public PHJointDesc{
	double	lower, upper;	///< ͈. lower < upper̂ƂɗLƂȂ
	double	spring;			///< olW
	double  origin;			///< ol_
	double  damper;			///< _pW
	double	torque;			///< [^gN

	PHJoint1DDesc():lower(0.0), upper(0.0), spring(0.0), origin(0.0), damper(0.0), torque(0.0){}
};

/// qW̃fBXNv^
struct PHHingeJointDesc : public PHJoint1DDesc{
	PHHingeJointDesc(){
		type = HINGEJOINT;
	}
};

/// XC_̃fBXNv^
struct PHSliderJointDesc : public PHJoint1DDesc{
	PHSliderJointDesc(){
		type = SLIDERJOINT;
	}
};

/// pX1̓_
struct PHPathPoint{
	double	s;			///< XJ[p[^
	Posed	pose;		///< ʒuƌX
};
/// pX̃fBXNv^
struct PHPathDesc{
	std::vector<PHPathPoint> points;	///< pX̓_
	bool bLoop;							///< trueȂ΃[vpXCfalseȂ΃I[vpXDftHgfalseD
	PHPathDesc():bLoop(false){}
};

/// pXWCg̃fBXNv^
struct PHPathJointDesc : public PHJoint1DDesc{
	PHPathJointDesc(){
		type = PATHJOINT;
	}
};

/// {[WCg̃fBXNv^
struct PHBallJointDesc : public PHJointDesc{
	double	max_angle;		///< ~̉͈
	Vec3d	torque;			///< [^gN
	PHBallJointDesc(){
		type = BALLJOINT;
		max_angle = 0.0;
	}
};

/// ol_p̃fBXNv^
struct PHSpringDesc : public PHConstraintDesc{
	Vec3d spring;		///< olW
	Vec3d damper;		///< _pW
	PHSpringDesc(){
		type = SPRING;
	}
};

/// S̃C^tF[X
struct PHConstraintIf : public SceneObjectIf{
	IF_DEF(PHConstraint);

	/** @brief SL/
		@param bEnable trueȂΗLCfalseȂΖ
	 */
	virtual void Enable(bool bEnable = true) = 0;

	/** @brief L擾
		@return LȂtrue, Ȃfalse
	 */
	virtual bool IsEnabled() = 0;

	/** @brief S̎ނ擾
		@return S̎
	 */
	virtual PHConstraintDesc::ConstraintType GetConstraintType() = 0;
};

/// ڐG_S̃C^tF[X
struct PHContactPointIf : public PHConstraintIf{
	IF_DEF(PHContactPoint);
};

/// ֐߂̃C^tF[X
struct PHJointIf : public PHConstraintIf{
	IF_DEF(PHJoint);
};

/// 1֐߂̃C^tF[X
struct PHJoint1DIf : public PHJointIf{
	IF_DEF(PHJoint1D);

	/** @brief ͈͂ݒ肷
		@param lower ͈͂̉
		@param upper ͈͂̏
	 */
	virtual void	SetRange(double lower, double upper) = 0;
	
	/** @brief ͈͂擾
		@param lower ͈͂̉擾ϐւ̎Q
		@param upper ͈͂̏擾ϐւ̎Q
	 */
	virtual void	GetRange(double& lower, double& upper) = 0;

	/** @brief [^gNݒ肷
		@param torque [^gN
	 */
	virtual void	SetMotorTorque(double t) = 0;

	/** @brief [^gN擾
		@return [^gN
	 */
	virtual double	GetMotorTorque() = 0;

	//virtual void	SetDesiredPosition(double p, double t) = 0;	/// ڕWψʂݒ肷
	//virtual double	GetDesiredPosition() = 0;				/// ڕWψʂ擾

	/** @brief ڕWxݒ肷
		@param vel ڕWx
	 */
	virtual void	SetDesiredVelocity(double v) = 0;

	/** @brief ڕWx擾
		@return ڕWx
	 */
	virtual double	GetDesiredVelocity() = 0;

	/** @brief olWݒ肷
		@param spring olW
	 */
	virtual void	SetSpring(double spring) = 0;

	/** @brief olW擾
		@return olW
	 */
	virtual double	GetSpring() = 0;

	/** @brief oľ_ݒ肷
		@param origin ol_

		ol_Ƃ̓ol̗͂0ƂȂ֐ߕψʂ̂ƁD
	 */
	virtual void	SetSpringOrigin(double origin) = 0;

	/** @brief oľ_擾
		@return ol_
	 */
	virtual double	GetSpringOrigin() = 0;

	/** @brief _pWݒ肷
		@param damper _pW
	 */
	virtual void	SetDamper(double damper) = 0;

	/** @brief _pW擾
		@return _pW
	 */
	virtual double	GetDamper() = 0;

	/** @brief ֐ߕψʂ擾
		@return ֐ߕψ
	 */
	virtual double	GetPosition() = 0;

	/** @brief ֐ߑx擾
		@return ֐ߑx
	 */
	virtual double	GetVelocity() = 0;
};

/// qW̃C^tF[X
struct PHHingeJointIf : public PHJoint1DIf{
	IF_DEF(PHHingeJoint);
};

/// XC_̃C^tF[X
struct PHSliderJointIf : public PHJoint1DIf{
	IF_DEF(PHSliderJoint);
};

/// pX̃C^tF[X
struct PHPathIf : public SceneObjectIf{
	IF_DEF(PHPath);
	virtual void AddPoint(double s, const Posed& pose) = 0;
	virtual void SetLoop(bool bOnOff = true) = 0;
	virtual bool IsLoop() = 0;
};

/// pXWCg̃C^tF[X
struct PHPathJointIf : public PHJoint1DIf{
	IF_DEF(PHPathJoint);
	virtual void SetPosition(double q)=0;
};

/// {[WCg̃C^tF[X
struct PHBallJointIf : public PHConstraintIf{
	IF_DEF(PHBallJoint);

	/** @brief őpxݒ肷
		@param angle őpx
	 */
	virtual void SetMaxAngle(double angle) = 0;

	/** @brief őpx擾
		@return őpx
	 */
	virtual double GetMaxAngle() = 0;

	/** @brief [^gNݒ肷
		@param torque [^gN
	 */
	virtual void SetMotorTorque(const Vec3d& torque)=0;

	/** @brief [^gN擾
		@return [^gN
	 */
	virtual Vec3d GetMotorTorque()=0;

	/** @brief ֐ߕψʂ擾
		@return ֐ߕψ
	 */
	virtual Quaterniond	GetPosition() = 0;

	/** @brief ֐ߑx擾
		@return ֐ߑx
	 */
	virtual Vec3d GetVelocity() = 0;

};

/// ol_p̃C^tF[X
struct PHSpringIf : public PHConstraintIf{
	IF_DEF(PHSpring);
	/** @brief olWݒ肷
		@param spring olW
	 */
	virtual void SetSpring(const Vec3d& spring)=0;

	/** @brief olW擾
		@return olW
	 */
	virtual Vec3d GetSpring()=0;

	/** @brief _pWݒ肷
		@param damper _pW
	 */
	virtual void SetDamper(const Vec3d& damper)=0;

	/** @brief _pW擾
		@return _pW
	 */
	virtual Vec3d GetDamper()=0;

};

//@}
//@}

}

#endif
