/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef FIFILE_H
#define FIFILE_H

#include <SprFileIO.h>
#include <Foundation/UTTypeDesc.h>
#include "FINodeHandler.h"

namespace Spr{;
class FILoadContext;
class FISaveContext;


/**	t@C[_[EZ[o[
*/
class FIFile:public InheritObject<FIFileIf, Object>{
protected:
	///	t@CɃZ[uEt@C烍[h^̏(UTTypeDesc)
	UTTypeDescDb typeDb;
	///	[hɎ[hƂ͕ʂ̏Kvȃm[h̃nh
	FINodeHandlers handlers;
public:
	OBJECT_DEF_ABST(FIFile);
	virtual bool Load(ObjectIfs& objs, const char* fn);
	virtual void Load(FILoadContext* fc);
	virtual bool Save(const ObjectIfs& objs, const char* fn);
	virtual void Save(const ObjectIfs& objs, FISaveContext* sc);
	/**	m[h̃[hD
		[hDescIuWFNg쐬D
		IuWFNg̍쐬́CeIuWFNgCreateObject()Ce̐eCreateObject()
		cCreateObject()ɌĂŁC쐬łƂŎ~߂D
		ǂ̃IuWFNg쐬łȂꍇ́CCreateSdk()ĂяoD	*/
	void LoadNode(FILoadContext* fc);
	///	m[h̃[h̏I
	void LoadEndNode(FILoadContext* fc);
	///	ubN(gݗČ^)ɓ
	void LoadEnterBlock(FILoadContext* fc);
	///	ubN(gݗČ^)o
	void LoadLeaveBlock(FILoadContext* fc);

protected:
	///	[h̎
	virtual void LoadImp(FILoadContext* fc)=0;
	
	///	m[h̃Z[u
	void SaveNode(FISaveContext* sc, ObjectIf* obj);
	///	ubÑZ[u
	void SaveBlock(FISaveContext* sc);

	///	ۑ̃nh[
	//@{
	///	t@CJn̏
	virtual void OnSaveFileStart(FISaveContext* sc){}
	///	t@CȈ
	virtual void OnSaveFileEnd(FISaveContext* sc){}
	///	m[hJn̏
	virtual void OnSaveNodeStart(FISaveContext* sc){}
	///	m[hȈ
	virtual void OnSaveNodeEnd(FISaveContext* sc){}
	///	f[^Jn̏
	virtual void OnSaveDataStart(FISaveContext* sc){}
	///	f[^Ȉ
	virtual void OnSaveDataEnd(FISaveContext* sc){}
	///	qIuWFNg̕ۑJn̏
	virtual void OnSaveChildStart(FISaveContext* sc){}
	///	qIuWFNg̕ۑȈ
	virtual void OnSaveChildEnd(FISaveContext* sc){}

	///	ubNJn̏
	virtual void OnSaveBlockStart(FISaveContext* sc){}
	///	ubNȈ
	virtual void OnSaveBlockEnd(FISaveContext* sc){}
	///	tB[hJn̏
	virtual void OnSaveFieldStart(FISaveContext* sc, int nElements){}
	///	tB[hȈ
	virtual void OnSaveFieldEnd(FISaveContext* sc, int nElements){}
	///	vfJn̏
	virtual void OnSaveElementStart(FISaveContext* sc, int pos, bool last){}
	///	vfȈ
	virtual void OnSaveElementEnd(FISaveContext* sc, int pos, bool last){}
	///	booll̕ۑ
	virtual void OnSaveBool(FISaveContext* sc, bool val){}
	///	intl̕ۑ
	virtual void OnSaveInt(FISaveContext* sc, int val){}
	///	reall̕ۑ
	virtual void OnSaveReal(FISaveContext* sc, double val){}
	///	stringl̕ۑ
	virtual void OnSaveString(FISaveContext* sc, UTString val){}
	virtual void OnSaveRef(FISaveContext* sc){}
	//@}

};

template <class intf, class base>
struct InheritFIFile:public InheritObject<intf, base>{
	virtual bool Load(ObjectIfs& objs, const char* fn){ return base::Load(objs, fn); }
	virtual bool Save(const ObjectIfs& objs, const char* fn){ return base::Save(objs, fn); }
};


}
#endif
