/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef SCENE_H
#define SCENE_H
#include <SprFoundation.h>
#include <set>
#include <map>

namespace Spr {;


class ObjectNamesLess{
public:
	bool operator () (const NamedObject* o1, const NamedObject* o2) const;
};
/**	OƃV[Ot̃IuWFNg̑Ή\
	OL[ɂZbgpӂCÖӐ̕ۏ؂ƃIuWFNg
	D	*/
class ObjectNames:public std::set<NamedObject*, ObjectNamesLess >{
public:
	typedef std::map<UTString, UTString> TNameMap;
	class ObjectKey:public NamedObject{
	public:
		ObjectKey();
		~ObjectKey();
		DEF_UTTYPEINFODEF(ObjectKey);
	};
	TNameMap nameMap;
	static ObjectKey key;
	void Print(std::ostream& os) const;

	///	OIuWFNg擾
	typedef std::pair<iterator, iterator> range_type;
	NamedObject* Find(UTString name, UTString cn="") const {
		key.name = name;
/*		setȂ̂ɂȂȂƂĂȂD
		UTString className = cn;
		key.typeInfo.className = className.c_str();
		const_iterator lb = lower_bound(&key);
		if (lb==end()) return NULL;
		ObjectNamesLess less;
		if (less(*lb, &key)) return NULL;		//	̂Ȃꍇ
		const_iterator it = lb;
		++it;
		if (it == end()) return *lb;
		if(less(&key, *it)) return *lb;		//	̂1
		return NULL;						//	̂Lꍇ
*/
		iterator it = ((ObjectNames*)this)->find(&key);
		if (it == end()) return NULL;
		NamedObject* obj = *it;
		if (obj->GetTypeInfo()->Inherit(cn.c_str())) return obj;
		return NULL;
	}
/*		set  range Ƃ͕̂ρD
	range_type Range(UTString name, UTString cn=""){
		key.name = name;
		UTString className = cn;
		key.typeInfo.className = className.c_str();
		iterator lb = lower_bound(&key);
		if (lb==end()) return range_type(end(), end());
		ObjectNamesLess less;
		iterator it = lb;
		while(it != end() && !less(*it, &key)){
			++it;
		}
		return range_type(lb, it);
	}
*/
	/**	IuWFNg̒ǉC
		ÔȂIuWFNg͒ǉłȂD̏ꍇ false ԂD
		ǉɐ trueD łɓo^Ăꍇ false ԂD
		OԂꍇ́CǉIuWFNg̖OύXD	*/
	bool Add(NamedObject* obj);
	///	IuWFNg̍폜
	bool Del(NamedObject* key){
		iterator it = find(key);
		if (it==end()) return false;
		erase(it);
		return true;
	}
	///	IuWFNg̍폜
	bool Del(UTString name){
		NamedObject key;
		key.name = name;
		key.AddRef();
		iterator it = find(&key);
		key.DelRef();
		if (it==end()) return false;
		erase(it);
		return true;
	}
};
inline std::ostream& operator << (std::ostream& os, const ObjectNames& ns){
	ns.Print(os); return os;
}

class SPR_DLL NameManager:public InheritNamedObject<NameManagerIf, NamedObject>{
public:
	OBJECT_DEF(NameManager);
	NameManager();
protected:
	/*	OƃIuWFNg̑Ή\  */
	ObjectNames names;
	
	///@name	OԂ\邽߂̃c[
	//@{
	///
	typedef std::vector<NameManager*> NameManagers;
	///	q̖O
	NameManagers childManagers;
	///	[gNameManager
	static NameManager theRoot;
	//@}

public:
	static NameManager* GetRoot(){ return &theRoot; }
	virtual void SetNameManager(NameManager* s);
	void AddChildManager(NameManager* c);
	void DelChildManager(NameManager* c);
	///	^ƖOIuWFNg擾
	template <class T> void FindObject(UTRef<T>& t, UTString name){
		T* p;
		FindObject(p, name);
		t = p;
	}
	template <class T> void FindObject(T*& t, UTString name){
		NamedObject* p = FindObject(name, GETCLASSNAMES(T));
		t = DCAST(T, p);
	}
	virtual NamedObjectIf* FindObject(UTString name, const char* cls=NULL);

	typedef ObjectNames::iterator SetIt;
//	typedef std::pair<SetIt, SetIt> SetRange;
//	SetRange RangeObject(UTString n){ return names.Range(n); }
	
	ObjectNames::TNameMap& GetNameMap(){ return names.nameMap; }

	///	O\CQƂĂȂIuWFNg폜D
	void ClearName();

	///	fobOp
	void Print(std::ostream& os) const;
	friend class NamedObject;
protected:
	NamedObjectIf* FindObjectExact(UTString name, const char* cls=NULL);
};
template <class intf, class base>
struct InheritNameManager:public InheritNamedObject<intf, base>{
	virtual NamedObjectIf* FindObject(UTString name, const char* cls=NULL){
		return base::FindObject(name, cls);
	}
};

/**	V[Ot̃gbvm[hDE_D
	_ƃV[Ot̊֌W[߁C
	_ႤƃV[Ot͂̂܂܎gpłȂD
	V[Ot́CƂ΃_OpenGLȂCdisplayList
	Ă邵CD3DȂ ID3DXMeshĂD
	OpenGL̃V[OtD3Dɕϊ邽߂ɂ́CxDocument
	Z[uāCD3D`Ń[hȂ΂ȂȂD	*/
class SPR_DLL Scene:public InheritNameManager<SceneIf, NameManager>{
	OBJECT_DEF(Scene);
public:
	///	RXgN^
	Scene();
	virtual ~Scene(){Clear();}
	///	V[ɂD
	void Clear();
};
template <class intf, class base>
struct InheritScene:public InheritNameManager<intf, base>{
};

}
#endif
