/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#include "Graphics.h"
#ifdef USE_HDRSTOP
#pragma hdrstop
#endif
#ifdef _MSC_VER
# include <WinBasis/WinBasis.h>
# include <windows.h>
#endif
#include <GL/glut.h>



namespace Spr {;
//----------------------------------------------------------------------------
//	GRDeviceGL
IF_OBJECT_IMP(GRDeviceGL, GRDevice);
/// ݒ
void GRDeviceGL::Init(){
	nLights = 0;
	glDrawBuffer(GL_BACK);
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glEnable(GL_BLEND);
//	glEnable(GL_CULL_FACE);
	glDisable(GL_CULL_FACE);
	SetVertexFormat(GRVertexElement::vfP3f);
	
	// _s̐ݒ
	viewMatrix.Pos() = Vec3f(0.0, 0.0, 1.0);	                        // eye
	viewMatrix.LookAtGL(Vec3f(0.0, 0.0, 0.0), Vec3f(0.0, 1.0, 0.0));	// center, up 
	viewMatrix = viewMatrix.inv();			
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(viewMatrix);
}
/// obt@NA
void GRDeviceGL::ClearBuffer(){
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClearDepth(1.0); 
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}
/// _O̊JnOɌĂԊ֐
void GRDeviceGL::BeginScene(){
  /// GLŃ_OꍇAɏKvȂ
}
/// _ȌIɌĂԊ֐
void GRDeviceGL::EndScene(){
  /// _uobt@[hAJgEBhẼobt@s
	glutSwapBuffers();
}
/// fs
void GRDeviceGL::MultModelMatrix(const Affinef& afw){	
	glMatrixMode(GL_MODELVIEW);
	glMultMatrixf(afw);
}
/// fs̍sX^bNPush
void GRDeviceGL::PushModelMatrix(){
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
}
/// fs̍sX^bNPop
void GRDeviceGL::PopModelMatrix(){
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}
/// fsݒ
void GRDeviceGL::SetModelMatrix(const Affinef& afw){
	modelMatrix = afw;		// fs̕ۑ
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(viewMatrix * modelMatrix);
}
///	_sݒ
void GRDeviceGL::SetViewMatrix(const Affinef& afv){   
	viewMatrix  = afv;		// _s̕ۑ
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(viewMatrix);
}
/// esݒ
void GRDeviceGL::SetProjectionMatrix(const Affinef& afp){  
	projectionMatrix = afp;
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projectionMatrix);
	glMatrixMode(GL_MODELVIEW);
}

/// _Ww肵ăv~eBu`
void GRDeviceGL::SetVertexFormat(const GRVertexElement* e){
	if (e == GRVertexElement::vfP3f) {
		vertexFormatGl = GL_V3F; 
		vertexSize = sizeof(float)*3;
		vertexColor = false;
	}else if (e == GRVertexElement::vfC4bP3f){
		vertexFormatGl = GL_C4UB_V3F;
		vertexSize = sizeof(float)*3+sizeof(char)*4;
		vertexColor = true;
	}else if (e == GRVertexElement::vfN3fP3f){
		vertexFormatGl = GL_N3F_V3F;
		vertexSize = sizeof(float)*6;
		vertexColor = false;
	}else if (e == GRVertexElement::vfC4fN3fP3f){
		vertexFormatGl = GL_C4F_N3F_V3F;
		vertexSize = sizeof(float)*10;
		vertexColor = true;
	}else if (e == GRVertexElement::vfT2fP3f){
		vertexFormatGl = GL_T2F_V3F;
		vertexSize = sizeof(float)*5;
		vertexColor = false;
	}else if (e == GRVertexElement::vfT2fC4bP3f){
		vertexFormatGl = GL_T2F_C4UB_V3F;
		vertexSize = sizeof(float)*5 + sizeof(char)*4;
		vertexColor = true;
	}else if (e == GRVertexElement::vfT2fN3fP3f){
		vertexFormatGl = GL_T2F_N3F_V3F;
		vertexSize = sizeof(float)*8;
		vertexColor = false;
	}else if (e == GRVertexElement::vfT2fC4fN3fP3f){
		vertexFormatGl = GL_T2F_C4F_N3F_V3F;
		vertexSize = sizeof(float)*12;
		vertexColor = true;
	}else {
		vertexFormatGl = 0;
		vertexSize = 0;
		vertexColor = false;
		assert(0);
	}
}
/// _Ww肵ăv~eBu`
void GRDeviceGL::SetVertexShader(void* s){
	assert(0);	//	To Be Implemented
}

/// _Ww肵ăv~eBu`
void GRDeviceGL::DrawDirect(TPrimitiveType ty, void* vtx, size_t count, size_t stride){
	GLenum mode = GL_TRIANGLES;
	switch(ty) {
		case POINTS:			mode = GL_POINTS;			break;
		case LINES:				mode = GL_LINES;			break;
		case LINE_STRIP:		mode = GL_LINE_STRIP;		break;
		case TRIANGLES:			mode = GL_TRIANGLES;		break;
		case TRIANGLE_STRIP:	mode = GL_TRIANGLE_STRIP;	break;
		case TRIANGLE_FAN:		mode = GL_TRIANGLE_FAN;		break;
		case QUADS:				mode = GL_QUADS;			break;
		default:				/* DO NOTHING */			break;
	}
	if (!stride) stride = vertexSize;
	if (vertexColor) glEnable(GL_COLOR_MATERIAL);
	else glDisable(GL_COLOR_MATERIAL);
	SetMaterial(currentMaterial);
	glInterleavedArrays(vertexFormatGl, stride, vtx);
	glDrawArrays(mode, 0, count);
}
/// _WƃCfbNXw肵ăv~eBu`
void GRDeviceGL::DrawIndexed(TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride){
	GLenum mode = GL_TRIANGLES;
	switch(ty) {
		case POINTS:			mode = GL_POINTS;			break;
		case LINES:				mode = GL_LINES;			break;
		case LINE_STRIP:		mode = GL_LINE_STRIP;		break;
		case TRIANGLES:			mode = GL_TRIANGLES;		break;
		case TRIANGLE_STRIP:	mode = GL_TRIANGLE_STRIP;	break;
		case TRIANGLE_FAN:		mode = GL_TRIANGLE_FAN;		break;
		case QUADS:				mode = GL_QUADS;			break;
		default:				/* DO NOTHING */			break;
	}
	if (!stride) stride = vertexSize;
	if (vertexColor) glEnable(GL_COLOR_MATERIAL);
	else glDisable(GL_COLOR_MATERIAL);
	SetMaterial(currentMaterial);
	glInterleavedArrays(vertexFormatGl, stride, vtx);
	glDrawElements(mode, count, GL_UNSIGNED_INT, idx);
}
int GRDeviceGL::CreateList(TPrimitiveType ty, void* vtx, size_t count, size_t stride){
	int list = glGenLists(1);
	glNewList(list, GL_COMPILE);
	DrawDirect(ty, vtx, count, stride);
	glEndList();
	return list;
}
int GRDeviceGL::CreateIndexedList(TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride){	
	int list = glGenLists(1);
	glNewList(list, GL_COMPILE);
	DrawIndexed(ty, idx, vtx, count, stride);
	glEndList();
	return list;
}
void GRDeviceGL::DrawList(int list){
	glCallList(list);
}
void GRDeviceGL::ReleaseList(int list){
	glDeleteLists(list, 1);
}
/// 3eLXg̕`iGLI[łfont͎wȂj .. Vec2f pos
void GRDeviceGL::DrawFont(Vec2f pos, const std::string str){
	glDisable(GL_LIGHTING);
	glRasterPos2fv(pos);
	std::string::const_iterator iter;
	for (iter = str.begin(); iter != str.end();	++iter) {
		glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *iter);
	}
	glEnable(GL_LIGHTING);
}
/// 3eLXg̕`iGLI[łfont͎wȂj.. Vec3f pos
void GRDeviceGL::DrawFont(Vec3f pos, const std::string str){
	glDisable(GL_LIGHTING);
	glRasterPos3fv(pos);
	std::string::const_iterator iter;
	for (iter = str.begin(); iter != str.end();	++iter) {
		glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *iter);
	}
	glEnable(GL_LIGHTING);
}
/// 3eLXg̕` ... Vec2f pos
void GRDeviceGL::DrawFont(Vec2f pos, const std::string str, const GRFont& font){
/// VCł̂fontT|[gB
#ifdef _MSC_VER	
	GLuint	base = 0;						// DisplayListJnwWԍ
	bool	findFont = false;				// fontLisť
	GLsizei	range	 = 256;					//	fBXvCXg̐
	unsigned long color=font.color;			//	tHg̐F

	if (fontList.size() > 0) {		// fontListłȂꍇ́AfontLisťKv
		// XgATCY10ȏ̏ꍇA擪폜	
		if (fontList.size() > 10) {		
			fontList.erase(fontList.begin());	
		}
		// fontList
		std::map<unsigned int, GRFont>::iterator itr = fontList.begin();
		while((itr != fontList.end()) && (findFont == false)) {
			if (((*itr).second.height == font.height)
				&& ((*itr).second.width == font.width)
				&& ((*itr).second.weight == font.weight)
				&& ((*itr).second.bItalic == font.bItalic)
				&& ((strcmp((*itr).second.face.c_str(),font.face.c_str()))==0)
				&& ((*itr).second.color == font.color)){
					findFont = true;
					base = (*itr).first;
					color = (*itr).second.color;
			} 
			++itr;
		}
	}
	if (findFont == false ) {		// VKo^̏ꍇ
		// Ŏw肳ꂽ_tHg쐬
		HFONT		hFont;			// tHgnh
		HFONT		hOldFont;	
		fontBase = glGenLists(range);
		// tHg̍쐬
		hFont = CreateFont(font.height,						//	tHg̍
							font.width,						//	ϕ
							0,								//	X̊px
							0,								//	eX̊px
							font.weight,					//	tHg̑
							font.bItalic,					//	C^bN
							FALSE,							//	A_[C
							FALSE,							//	łt
							ANSI_CHARSET,					//	tHgZbg
							OUT_DEFAULT_PRECIS,				//	o͐x
							CLIP_DEFAULT_PRECIS,			//	NbsOx
							ANTIALIASED_QUALITY,			//	o͕i
							FF_DONTCARE | DEFAULT_PITCH,	//	sb`ƃt@~(ԊuƏ)
							font.face.c_str());				//	^CvtFCX
		
		assert(hFont);
		HDC hDC = GetDC(0);
		// 0256̃R[h̕ADisplayListbaseԖڂo^.
		// wglUseFontBitmaps()֐AgpāAtHgfBXvCXgɊĂ.
		hOldFont = (HFONT)SelectObject(hDC, hFont);			
		wglUseFontBitmaps(hDC, 0, range, fontBase);		
		SelectObject(hDC, hOldFont);
		DeleteObject(hFont);		
		// fontList֓o^
		fontList.insert(std::map<unsigned int, GRFont>::value_type(fontBase, font));	
		base = fontBase;	
	}

	glDisable(GL_LIGHTING);
	glColor3f(((font.color >> 16) & 0xFF) / 255.0,
			  ((font.color >>  8) & 0xFF) / 255.0,
			  ((font.color >>  0) & 0xFF) / 255.0);
	glRasterPos2fv(pos);
	glPushAttrib(GL_LIST_BIT);
	glListBase(base);											// 	fBXvCXgn.	
	glCallLists(str.size(), GL_UNSIGNED_BYTE, str.c_str());		// n.
	glPopAttrib();
	glEnable(GL_LIGHTING);
#else	// _MSC_VER	
	// VCȊO̊ł́AfontT|[gȂB
	DrawFont(pos, str);
#endif	
}
/// 3eLXg̕`  ... Vec3f pos
void GRDeviceGL::DrawFont(Vec3f pos, const std::string str, const GRFont& font){
/// VCł̂fontT|[gB
#ifdef _MSC_VER		
	GLuint	base = 0;						// DisplayListJnwWԍ
	bool	findFont = false;				// fontLisť
	GLsizei	range	 = 256;					//	fBXvCXg̐
	unsigned long color=font.color;			//	tHg̐F

	if (fontList.size() > 0) {		// fontListłȂꍇ́AfontLisťKv
		// XgATCY10ȏ̏ꍇA擪폜	
		if (fontList.size() > 10) {		
			fontList.erase(fontList.begin());	
		}
		// fontList
		std::map<unsigned int, GRFont>::iterator itr = fontList.begin();
		while((itr != fontList.end()) && (findFont == false)) {
			if (((*itr).second.height == font.height)
				&& ((*itr).second.width == font.width)
				&& ((*itr).second.weight == font.weight)
				&& ((*itr).second.bItalic == font.bItalic)
				&& ((strcmp((*itr).second.face.c_str(),font.face.c_str()))==0)
				&& ((*itr).second.color == font.color)){
					findFont = true;
					base = (*itr).first;
					color = (*itr).second.color;
			} 
			++itr;
		}
	}
	if (findFont == false ) {		// VKo^̏ꍇ
		// Ŏw肳ꂽ_tHg쐬
		HFONT		hFont;			// tHgnh
		HFONT		hOldFont;	
		fontBase = glGenLists(range);
		// tHg̍쐬
		hFont = CreateFont(font.height,						//	tHg̍
							font.width,						//	ϕ
							0,								//	X̊px
							0,								//	eX̊px
							font.weight,					//	tHg̑
							font.bItalic,					//	C^bN
							FALSE,							//	A_[C
							FALSE,							//	łt
							ANSI_CHARSET,					//	tHgZbg
							OUT_DEFAULT_PRECIS,				//	o͐x
							CLIP_DEFAULT_PRECIS,			//	NbsOx
							ANTIALIASED_QUALITY,			//	o͕i
							FF_DONTCARE | DEFAULT_PITCH,	//	sb`ƃt@~(ԊuƏ)
							font.face.c_str());				//	^CvtFCX
		
		assert(hFont);
		HDC hDC = GetDC(0);
		// 0256̃R[h̕ADisplayListbaseԖڂo^.
		// wglUseFontBitmaps()֐AgpāAtHgfBXvCXgɊĂ.
		hOldFont = (HFONT)SelectObject(hDC, hFont);			
		wglUseFontBitmaps(hDC, 0, range, fontBase);		
		SelectObject(hDC, hOldFont);
		DeleteObject(hFont);		
		// fontList֓o^
		fontList.insert(std::map<unsigned int, GRFont>::value_type(fontBase, font));	
		base = fontBase;	
	}

	glDisable(GL_LIGHTING);
	glColor3f(((font.color >> 16) & 0xFF) / 255.0,
			  ((font.color >>  8) & 0xFF) / 255.0,
			  ((font.color >>  0) & 0xFF) / 255.0);
	glRasterPos3fv(pos);
	glPushAttrib(GL_LIST_BIT);
	glListBase(base);											// 	fBXvCXgn.	
	glCallLists(str.size(), GL_UNSIGNED_BYTE, str.c_str());		// n.
	glPopAttrib();
	glEnable(GL_LIGHTING);
#else	// _MSC_VER	
	// VCȊO̊ł́AfontT|[gȂB
	DrawFont(pos, str);
#endif	
}
/// `̍ގ̐ݒ
void GRDeviceGL::SetMaterial(const GRMaterialDesc& mat){
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   mat.ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   mat.diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  mat.specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  mat.emissive);
	glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, mat.power);
	currentMaterial = mat;
}
/// `悷_Ȇ̐ݒ
void GRDeviceGL::SetLineWidth(float w){
	glLineWidth(w);
}
/// X^bNPush
void GRDeviceGL::PushLight(const GRLightDesc& light){
	if (nLights < GL_MAX_LIGHTS) {
		glLightfv(GL_LIGHT0+nLights, GL_AMBIENT, light.ambient);
		glLightfv(GL_LIGHT0+nLights, GL_DIFFUSE, light.diffuse);
		glLightfv(GL_LIGHT0+nLights, GL_SPECULAR, light.specular);
		glLightfv(GL_LIGHT0+nLights, GL_POSITION, light.position);
		if (light.position.W()){
			// attenuation factor = 1/(GL_CONSTANT_ATTENUATION + GL_LINEAR_ATTENUATION * d + GL_QUADRATIC_ATTENUATION * d^2)
			glLightf(GL_LIGHT0+nLights, GL_CONSTANT_ATTENUATION, light.attenuation0);		// 	茸
			glLightf(GL_LIGHT0+nLights, GL_LINEAR_ATTENUATION, light.attenuation1);			//	`
			glLightf(GL_LIGHT0+nLights, GL_QUADRATIC_ATTENUATION, light.attenuation2);		//	2
			glLightfv(GL_LIGHT0+nLights, GL_SPOT_DIRECTION, light.spotDirection);
			glLightf(GL_LIGHT0+nLights, GL_SPOT_EXPONENT, light.spotFalloff);
			glLightf(GL_LIGHT0+nLights, GL_SPOT_CUTOFF, light.spotCutoff);
		}
		glEnable(GL_LIGHT0+nLights);
	}
	nLights ++;
}
/// X^bNPop
void GRDeviceGL::PopLight(){
	nLights--;
	if (nLights < GL_MAX_LIGHTS) glDisable(GL_LIGHT0+nLights);
}
/// fvXobt@ւ݂̏/֎~
void GRDeviceGL::SetDepthWrite(bool b){
	glDepthMask(b);
}
/// fvXeXgL/ɂ
void GRDeviceGL::SetDepthTest(bool b){
if (b) glEnable(GL_DEPTH_TEST);
	else glDisable(GL_DEPTH_TEST);
}
/// fvXobt@@ɗp锻w肷
void GRDeviceGL::SetDepthFunc(TDepthFunc f){
	GLenum dfunc = GL_LESS;
	switch(f){
		case DF_NEVER:		dfunc = GL_NEVER;		break;
		case DF_LESS:		dfunc = GL_LESS;		break;
		case DF_EQUAL:		dfunc = GL_EQUAL;		break;
		case DF_LEQUAL:		dfunc = GL_LEQUAL;		break;
		case DF_GREATER:	dfunc = GL_GREATER;		break;
		case DF_NOTEQUAL:	dfunc = GL_NOTEQUAL;	break;
		case DF_GEQUAL:		dfunc = GL_GEQUAL;		break;
		case DF_ALWAYS:		dfunc = GL_ALWAYS;		break;
		default:			/* DO NOTHING */		break;

	}
	glDepthFunc(dfunc);
}
/// At@ufBOL/ɂ
void GRDeviceGL::SetAlphaTest(bool b){
if (b) glEnable(GL_BLEND);
	else glDisable(GL_BLEND);
}
/// At@ufBÕ[hݒ(SRC̍W, DESTW)
void GRDeviceGL::SetAlphaMode(TBlendFunc src, TBlendFunc dest){
	TBlendFunc  bffac[2] = { src, dest };
	GLenum glfac[2];

	for (int iCnt=0; iCnt<2; ++iCnt){
		switch(bffac[iCnt]) {
			case BF_ZERO:				glfac[iCnt] = GL_ZERO;					break;
			case BF_ONE:				glfac[iCnt] = GL_ONE;					break;
			case BF_SRCCOLOR:			glfac[iCnt] = GL_SRC_COLOR;				break;
			case BF_INVSRCCOLOR:		glfac[iCnt] = GL_ONE_MINUS_SRC_COLOR;	break;
			case BF_SRCALPHA:			glfac[iCnt] = GL_SRC_ALPHA;				break;
			case BF_INVSRCALPHA:		glfac[iCnt] = GL_ONE_MINUS_SRC_ALPHA;	break;
			case BF_DESTALPHA:			glfac[iCnt] = GL_DST_ALPHA;				break;
			case BF_INVDESTALPHA:		glfac[iCnt] = GL_ONE_MINUS_DST_ALPHA;	break;
			case BF_DESTCOLOR:			glfac[iCnt] = GL_DST_COLOR;				break;
			case BF_INVDESTCOLOR:		glfac[iCnt] = GL_ONE_MINUS_DST_COLOR;	break;
			case BF_SRCALPHASAT:		glfac[iCnt] = GL_SRC_ALPHA_SATURATE;	break;
			case BF_BOTHINVSRCALPHA:	glfac[iCnt] = 0;						break;
			case BF_BLENDFACTOR:		glfac[iCnt] = 0;						break;
			case BF_INVBLENDFACTOR:		glfac[iCnt] = 0;						break;
			default:					/* DO NOTHING */						break;
		}
	}
	glBlendFunc(glfac[0], glfac[1]);

}
void GRDeviceGL::SetViewport(Vec2f pos, Vec2f sz){
	glViewport((int)pos.x , (int)pos.y, (int)sz.x, (int)sz.y);
}

}	//	Spr

