/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHCONSTRAINT_H
#define PHCONSTRAINT_H

#include <SprPhysics.h>

#define SUBMAT(r, c, h, w) sub_matrix(PTM::TSubMatrixDim<r, c, h, w>())
#define SUBVEC(o, l) sub_vector(PTM::TSubVectorDim<o, l>())

namespace Spr{;

typedef PTM::TMatrixRow<6, 6, double> Matrix6d;
typedef PTM::TMatrixRow<6, 3, double> Matrix63d;
typedef PTM::TVector<6, double> Vec6d;

/// ̂̏
struct PHSolidInfoForLCP : public PHSolidInfo{
	double		minv;		/// ʂ̋t
	Matrix3d	Iinv;		/// s̋ts
	Vec3d		f, t;		/// [JWł̊O
	Vec3d		v, w;		/// [JWł݂̌̑x
	Vec3d		dv0, dw0;	/// S͈ȊO̊O͂ɂ鑬xω
	Vec3d		dv, dw;		/// S͂ɂ鑬xω
	//Vec3d		dV, dW;		/// CorrectionɂړʁC]
	void SetupDynamics(double dt);
	void SetupCorrection();
	PHSolidInfoForLCP(PHSolid* s):PHSolidInfo(s){}
};

/// PHConstrainthNX̂߂̃[^
template<class inf, class base>
class InheritConstraint : public InheritSceneObject<inf, base>{
public:
	void Enable(bool bEnable = true){base::Enable(bEnable);}
	bool IsEnabled(){return base::IsEnabled();}
};

///
class PHConstraint : public InheritSceneObject<PHConstraintIf, SceneObject>{
public:
	OBJECT_DEF_ABST(PHConstraint);
	enum PHControlMode{
		MODE_TORQUE,
		MODE_POSITION,
		MODE_VELOCITY
	} mode;
	
	bool		bEnabled;			/// LĂꍇtrue
	bool		bFeasible;			/// ̍̂undynamicalȏꍇtrue

	PHSolidInfoForLCP* solid[2];
	Matrix3d	Rj[2];				/// êɒt֐߃t[
	Vec3d		rj[2];
	Matrix3d	Rjrel;				/// ֐߃t[Ԃ̈ʒu֌W
	Quaterniond	qjrel;
	Vec3d		rjrel;
	Vec3d		vjrel, wjrel;		/// ֐߃t[Ԃ̑Αx
	/// Jvv, Jvw : ê̑xCpx瑊Αxւ̃Rrs
	/// Jwv, Jww : ê̑xCpx瑊Ίpxւ̃Rrs
	/// Jqv, Jqw : ê̑xCpx瑊quaternion̎Ԕւ̃Rrs
	/**
				|	  v[0]	  w[0]	  v[1]	  w[1]
	  ----------+---------------------------------
		vjrel	|	Jvv[0]	Jvw[0]	Jvv[1]	Jvw[1]
		wjrel	|	Jwv[0]	Jww[0]	Jwv[1]	Jww[1]
		qjrel	|	Jqv[0]	Jqw[0]	Jqv[1]	Jqw[1]
	*/
	Matrix3d	Jvv[2], Jvw[2], Jwv[2], Jww[2];	//, Jqv[2], Jqw[2];
	Matrix3d	Tvv[2], Tvw[2], Twv[2], Tww[2];
	
	//int		dim_d, dim_c;	// S̎
	bool		constr[6];	// S鎩RxDtrueȂ΍SD

	//Matrix63d	Jdv[2], Jdw[2];	// Dynamicsp̍SRrs
	//Matrix63d	Tdv[2], Tdw[2];
	
	//Matrix63d	Jcv[2], Jcw[2];	// Correctionp̍SRrs
	//Matrix63d	Tcv[2], Tcw[2];

	Vec3d		fv, fw;	/// dynamicsɂ֐ߗ
	//Vec3d		Fv, Fq; /// correctionɂ֐ߗ

	//Vec6d		b, B;	// bxNgDdynamicspcorrectionp
	Vec3d		bv, bw;	/// dynamicsɂSx
	//Vec3d		bv_bias, bw_bias;	/// x邽߂bv, bw̃ItZbg
	//Vec3d		Bv, Bq; /// correctionɂS덷
	//Vec6d		Ad, Ac;	// As̑ΊpDdynamicspcorrectionp
	Vec3d		Av, Aw; //, Aq;

	virtual void SetDesc(const PHConstraintDesc& desc);
	virtual bool AddChildObject(ObjectIf* o);
	virtual void Enable(bool bEnable = true){bEnabled = bEnable;}
	virtual bool IsEnabled(){return bEnabled;}
	void Init();
	void CompJacobian(bool bCompAngular);
	void SetupDynamics(double dt, double correction_rate);
	void IterateDynamics();
	//void SetupCorrection(double dt, double max_error);
	//void IterateCorrection();
	//virtual void CompDof(){}					/// dim_v, dim_w, dim_qݒ肷
	//virtual void CompMotorForce(){}				/// fv, fwɃ[^ɂeݒ肷
	//virtual void CompConstraintJacobian()=0;
	virtual void CompBias(double dt, double correction_rate){}	/// 
	//virtual void CompError(double dt)=0;			/// Bv, Bqݒ肷
	virtual void Projection(double& f, int k){}
	//virtual void ProjectionCorrection(double& F, int k){}
	PHConstraint();
};
class PHConstraints : public std::vector< UTRef<PHConstraint> >{
public:
	void SetupDynamics(double dt, double correction_rate){
		for(iterator it = begin(); it != end(); it++)
			(*it)->SetupDynamics(dt, correction_rate);
	}
	/*void SetupCorrection(double dt, double max_error){
		for(iterator it = begin(); it != end(); it++)
			(*it)->SetupCorrection(dt, max_error);
	}*/
	void IterateDynamics(){
		for(iterator it = begin(); it != end(); it++)
			(*it)->IterateDynamics();
	}
	/*void IterateCorrection(){
		for(iterator it = begin(); it != end(); it++)
			(*it)->IterateCorrection();
	}*/
};

}

#endif
