/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHSOLID_H
#define PHSOLID_H

#include <SprPhysics.h>
#include <Foundation/Object.h>
#include <Physics/PHScene.h>
#include <Physics/PHEngine.h>

namespace Spr{;

///	oEfBO{bNX̎
class PHBBox{
	Vec3f bboxCenter;	///<	BBox̒S([Jn)
	Vec3f bboxExtent;	///<	BBox̑傫([Jn)
public:
	///	oEfBO{bNX̐ݒ
	void SetBBoxCenterExtent(Vec3f c, Vec3f e){
		bboxCenter = c;
		bboxExtent = e;
	}
	///	oEfBO{bNX̐ݒ
	void SetBBoxMinMax(Vec3f bmin, Vec3f bmax){
		bboxCenter = (bmin+bmax)*0.5f;
		bboxExtent = (bmax-bmin)*0.5f;
	}
	///	oEfBO{bNX̒ǉ
	void AddBBox(Vec3f bmin, Vec3f bmax){
		Vec3f bboxMin = GetBBoxMin();
		Vec3f bboxMax = GetBBoxMax();
		bboxMin.element_min(bmin);
		bboxMax.element_max(bmax);
		SetBBoxMinMax(bboxMin, bboxMax);
	}
	///	S
	Vec3f GetBBoxCenter(){ return bboxCenter; }
	///	傫
	Vec3f GetBBoxExtent(){ return bboxExtent; }
	///	[_
	Vec3f GetBBoxMin(){ return bboxCenter-bboxExtent; }
	///	傫[_
	Vec3f GetBBoxMax(){ return bboxCenter+bboxExtent; }
	///	^ꂽxNgƂ̓ςőƍŏ̓_
	void GetSupport(const Vec3f& dir, float& minS, float& maxS){
		Vec3f ext0( bboxExtent.X(),  bboxExtent.Y(),  bboxExtent.Z());
		Vec3f ext1(-bboxExtent.X(),  bboxExtent.Y(),  bboxExtent.Z());
		Vec3f ext2( bboxExtent.X(), -bboxExtent.Y(),  bboxExtent.Z());
		Vec3f ext3( bboxExtent.X(),  bboxExtent.Y(), -bboxExtent.Z());
		float d = abs(dir*ext0);
		float d1 = abs(dir*ext1);
		if (d < d1) d = d1;
		float d2 = abs(dir*ext2);
		if (d < d2) d = d2;
		float d3 = abs(dir*ext3);
		if (d < d3) d = d3;
		float c = dir * bboxCenter;
		minS = c-d;
		maxS = c+d;
	}
};


enum PHIntegrationMode{
		PHINT_NONE,				///	ϕȂ
		PHINT_ARISTOTELIAN,		///	f = mv
		PHINT_EULER,			///	IC[@
		PHINT_SIMPLETIC,		/// VvNeBbN@
		PHINT_ANALYTIC,			
		PHINT_RUNGEKUTTA2,		///	QQNb^@
		PHINT_RUNGEKUTTA4		///	SQNb^@
};


///	
class PHSolid : public InheritSceneObject<PHSolidIf, SceneObject>, public PHSolidDesc{
protected:
	bool		bUpdated;		///<	̃GWSolid̍XVǗ邽߂̃tO
	bool		bDynamical;		///<	͊wIǂ̃tO
	Matrix3d	inertia_inv;	///<	e\̋t(LocalnELbV)

	///	ϕ
	PHIntegrationMode integrationMode;

	//IC[̉^
	//s͑ΊpsOD
	Vec3d	Euler(const Matrix3d& I, const Vec3d& t, const Vec3d& w){
		return Vec3d(
			(t[0] - (I[2][2] - I[1][1]) * w.Y() * w.Z()) / I[0][0],
			(t[1] - (I[0][0] - I[2][2]) * w.Z() * w.X()) / I[1][1],
			(t[2] - (I[1][1] - I[0][0]) * w.X() * w.Y()) / I[2][2]);
	}
public:
	std::vector<CDShapeRefWithPose> shapes;
	PHBBox bbox;

	OBJECT_DEF(PHSolid);
	PHSolid(const PHSolidDesc& desc=PHSolidDesc(), SceneIf* s=NULL);

	void SetScene(SceneIf* s);
	CDShapeIf* CreateShape(const CDShapeDesc& desc);
	ObjectIf* CreateObject(const IfInfo* info, const void* desc);
	bool AddChildObject(ObjectIf* obj);
	size_t NChildObject() const{
		return shapes.size();
	}
	ObjectIf* GetChildObject(size_t pos) {
		return (CDShapeIf*)(shapes[pos].shape);
	}

	void CalcBBox();
	void GetBBoxSupport(const Vec3f& dir, float& minS, float& maxS);
	
	void		SetUpdated(bool set){bUpdated = set;}	///< 
	bool		IsUpdated(){return bUpdated;}			///< 
	void		Step();									///< i߂D
	
	void		AddForce(Vec3d f);						///< ͂ʒSɉ
	void		AddTorque(Vec3d t);						///< gN
	void		AddForce(Vec3d f, Vec3d r);				///< ͂ ʒur(Worldn) ɉ
	//邵GravityEnginegĂȂ̂Ŕp
	//void		AddForceLocal(Vec3d f, Vec3d r);		///< ͂ ʒur(Localn) ɉ
	Vec3d		GetForce() const {return force;}		///< ꂽ
	Vec3d		GetTorque() const {return torque;}		///< ꂽgN

	double		GetMass(){return mass;}					///< 
	double		GetMassInv(){return 1.0 / mass;}		///< ʂ̋t
	void		SetMass(double m){mass = m;}			///< ʂ̐ݒ
	void		SetMassInv(double minv){mass = 1.0 / minv;}
														///< ʂ̋t̐ݒ
	Matrix3d	GetInertia(){return inertia;}			///< e\
	Matrix3d	GetInertiaInv(){return inertia_inv;}	///< e\̋t
	void		SetInertia(const Matrix3d& I){			///< e\̐ݒ
		inertia = I;
		inertia_inv = I.inv();
	}
	void		SetInertiaInv(const Matrix3d& Iinv){	///< e\tŐݒ
		inertia_inv = Iinv;
		//inertia = Iinv.inv();
	}

	///	ϕ̎擾
	PHIntegrationMode GetIntegrationMode(){ return integrationMode; }
	///	ϕ̐ݒ
	void SetIntegrationMode(PHIntegrationMode m){ integrationMode=m; }

	Posed		GetPose() const { return pose; }
	void		SetPose(const Posed& p) { pose = p;}
	Vec3d		GetFramePosition() const {return pose.Pos();}
	void		SetFramePosition(const Vec3d& p){pose.Pos() = p;}
	Vec3d		GetCenterPosition() const {return pose*center;}
														///< dSʒu̎擾
	void		SetCenterPosition(const Vec3d& p){		///< dSʒu̐ݒ
		pose.Pos() = p - pose.Ori()*center;
	}

	///	̎擾
	Matrix3d	GetRotation() const { Matrix3d rv; pose.Ori().ToMatrix(rv); return rv; }
	///	̐ݒ
	void		SetRotation(const Matrix3d& r){
		pose.Ori().FromMatrix(r);
	}

	///	̎擾
	Quaterniond GetOrientation() const {return pose.Ori();}
	///	̐ݒ
	void		SetOrientation(const Quaterniond& q){
		pose.Ori() = q;
		Matrix3f m;
		pose.Ori().ToMatrix(m);
	}

	///	ʒS̑x̎擾
	Vec3d		GetVelocity() const {return velocity;}
	///	ʒS̑x̐ݒ
	void		SetVelocity(const Vec3d& v){velocity = v;}

	///	px̎擾
	Vec3d		GetAngularVelocity() const {return angVelocity;}
	///	px̐ݒ
	void		SetAngularVelocity(const Vec3d& av){angVelocity = av;}

	///	[Jt[猩Ĉ̎ʒSʒu̐ݒ
	Vec3d		GetCenter() const {return center;}
	///	[Jt[猩Ĉ̎ʒSʒu̎擾
	void		SetCenter(const Vec3d& c){center = c;}		

	///	Cӂ̈ʒuł̑x̎擾
	Vec3d		GetPointVelocity(Vec3f posW) const {
		return velocity + (angVelocity^(posW - pose*center));
	}

	///	̍̂ Spr::CDShape ̐
	int			NShape();
	///	̍̂ iԖڂ SPR::CDShape ̎擾
	CDShapeIf*	GetShape(int i);
	///	shape  ̍̂Spr::CDShapẽXg ̍ŌɒǉD
	void		AddShape(CDShapeIf* shape);
	///	̍̂ iԖڂ SPR::CDShape ̂̍̂猩p擾
	Posed		GetShapePose(int i);
	///	̍̂ iԖڂ SPR::CDShape ̂̍̂猩pݒ
	void		SetShapePose(int i, const Posed& pose);

	void		SetGravity(bool bOn);
	void		SetDynamical(bool bOn){bDynamical = bOn;}
	bool		IsDynamical(){return bDynamical;}

protected:
	ACCESS_DESC_STATE(PHSolid);
};

class PHSolids:public std::vector< UTRef<PHSolid> >{
public:
	UTRef<PHSolid> Erase(const PHSolid* s){
		iterator it = std::find(begin(), end(), s);
		if (it == end()) return NULL;
		UTRef<PHSolid> rv = *it;
		erase(it);
		return *it;
	}
	UTRef<PHSolid>* Find(const PHSolid* s){
		iterator it = std::find(begin(), end(), s);
		if (it == end()) return NULL;
		else return &*it;
	}
	UTRef<PHSolid>* Find(const PHSolid* s) const {
		return ((PHSolids*)this)->Find(s);
	}
};

/**	SolidێNXDSolid̍XVsD	*/
class PHSolidContainer:public PHEngine{
	OBJECT_DEF_NOIF(PHSolidContainer);
public:
	PHSolids solids;
	bool AddChildObject(ObjectIf* o);
	bool DelChildObject(ObjectIf* o);
	///
	int GetPriority() const {return SGBP_SOLIDCONTAINER;}
	/// solid̃Zbg
	void Reset();
	///	xʒuAxx̐ϕ
	virtual void Step();
	
	virtual void Clear(PHScene* s){ solids.clear(); }
	///	LĂsolid̐
	virtual size_t NChildObject() const { return solids.size(); }
	///	LĂsolid
	virtual ObjectIf* GetChildObject(size_t i){ return (PHSolidIf*)solids[i]; }

	PHSolidContainer();
};

/** SolidContainerResetĂԂ߂̃GW */
class PHSolidInitializer : public PHEngine{
	OBJECT_DEF_NOIF(PHSolidInitializer);
public:
	PHSolidContainer* container;
	
	///
	int GetPriority() const {return SGBP_SOLIDINITIALIZER;}
	///	updatedtOZbg
	virtual void Step();
};


}	//	namespace Spr
#endif
